# -*- coding:utf-8 -*-
"""
估值指标，季频
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
@copyright: baostock System & alpha.All Rights Reserved.
"""
import time
import zlib
import baostock.data.resultset as rs
import baostock.common.contants as cons
import baostock.util.stringutil as strUtil
import baostock.common.context as conx
import baostock.util.socketutil as sock
import baostock.data.messageheader as msgheader


def query_dividend_data(code, year=None, yearType="report"):
    """估值指标（季频）,股息分红
    @param code: 证券代码，不可为空
    @param year: 年份，为空时默认当前年份
    @param yearType: 年份类别，默认为"report":预案公告年份，可选项"operate":除权除息年份
    """

    data = rs.ResultData()

    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if year is None or year == "":
        year = time.strftime("%Y", time.localtime())
    if yearType is None or yearType == "":
        print("年份类别输入有误，请修改。")
        data.error_msg = "年份类别输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    year = str(year)
    if not year.isdigit():
        print("年份输入有误，请修改。")
        data.error_msg = "年份输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_dividend_data", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, year, yearType)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYDIVIDENDDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYDIVIDENDDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.year = body_arr[8]
        data.yearType = body_arr[9]
        data.setFields(body_arr[10])

    return data


def query_adjust_factor(code, start_date=None, end_date=None):
    """复权因子信息
    @param code: 证券代码，不可为空
    @param start_date: 起始除权除息日期，为空时默认2015-01-01）
    @param end_date: 终止除权除息日期，为空时默认当前时间
    """

    data = rs.ResultData()

    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if start_date is None or start_date == "":
        start_date = cons.DEFAULT_START_DATE
    if end_date is None or end_date == "":
        end_date = time.strftime("%Y-%m-%d", time.localtime())

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_adjust_factor", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, start_date, end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_ADJUSTFACTOR_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_ADJUSTFACTOR_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.start_date = body_arr[8]
        data.end_date = body_arr[9]
        data.setFields(body_arr[10])

    return data


def query_profit_data(code, year=None, quarter=None):
    """季频盈利能力
    @param code: 证券代码，不可为空
    @param year: 统计年份，为空时默认当前年
    @param quarter: 统计季度，为空时默认当前季度
    """
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if year is None or year == "":
        year = time.strftime("%Y", time.localtime())  # 当前年份
    if quarter is None or quarter == "":
        quarter = (int(time.strftime("%m", time.localtime())) + 2) // 3  # 当前季度
    elif quarter not in [1, 2, 3, 4] and quarter not in ["1", "2", "3", "4"]:
        print("季度填写错误，请检查。填写范围：1，2，3，4")
        data.error_msg = "季度填写错误，请检查。填写范围：1，2，3，4"
        data.error_code = cons.BSERR_PARAM_ERR
    year = str(year)
    if not year.isdigit():
        print("年份输入有误，请修改。")
        data.error_msg = "年份输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_profit_data", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, year, quarter)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_PROFITDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_PROFITDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.year = body_arr[8]
        data.quarter = body_arr[9]
        data.setFields(body_arr[10])
    return data


def query_operation_data(code, year=None, quarter=None):
    """季频营运能力
    @param code: 证券代码，不可为空
    @param year: 统计年份，为空时默认当前年
    @param quarter: 统计季度，为空时默认当前季度
    """
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if year is None or year == "":
        year = time.strftime("%Y", time.localtime())  # 当前年份
    if quarter is None or quarter == "":
        quarter = (int(time.strftime("%m", time.localtime())) + 2) // 3  # 当前季度
    elif quarter not in [1, 2, 3, 4] and quarter not in ["1", "2", "3", "4"]:
        print("季度填写错误，请检查。填写范围：1，2，3，4")
        data.error_msg = "季度填写错误，请检查。填写范围：1，2，3，4"
        data.error_code = cons.BSERR_PARAM_ERR
    year = str(year)
    if not year.isdigit():
        print("年份输入有误，请修改。")
        data.error_msg = "年份输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_operation_data", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, year, quarter)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_OPERATIONDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_OPERATIONDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.year = body_arr[8]
        data.quarter = body_arr[9]
        data.setFields(body_arr[10])
    return data


def query_growth_data(code, year=None, quarter=None):
    """季频成长能力
    @param code: 证券代码，不可为空
    @param year: 统计年份，为空时默认当前年
    @param quarter: 统计季度，为空时默认当前季度
    """
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if year is None or year == "":
        year = time.strftime("%Y", time.localtime())  # 当前年份
    if quarter is None or quarter == "":
        quarter = (int(time.strftime("%m", time.localtime())) + 2) // 3  # 当前季度
    elif quarter not in [1, 2, 3, 4] and quarter not in ["1", "2", "3", "4"]:
        print("季度填写错误，请检查。填写范围：1，2，3，4")
        data.error_msg = "季度填写错误，请检查。填写范围：1，2，3，4"
        data.error_code = cons.BSERR_PARAM_ERR
    year = str(year)
    if not year.isdigit():
        print("年份输入有误，请修改。")
        data.error_msg = "年份输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_growth_data", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, year, quarter)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYGROWTHDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYGROWTHDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.year = body_arr[8]
        data.quarter = body_arr[9]
        data.setFields(body_arr[10])
    return data


def query_dupont_data(code, year=None, quarter=None):
    """季频杜邦指数
    @param code: 证券代码，不可为空
    @param year: 统计年份，为空时默认当前年
    @param quarter: 统计季度，为空时默认当前季度
    """
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if year is None or year == "":
        year = time.strftime("%Y", time.localtime())  # 当前年份
    if quarter is None or quarter == "":
        quarter = (int(time.strftime("%m", time.localtime())) + 2) // 3  # 当前季度
    elif quarter not in [1, 2, 3, 4] and quarter not in ["1", "2", "3", "4"]:
        print("季度填写错误，请检查。填写范围：1，2，3，4")
        data.error_msg = "季度填写错误，请检查。填写范围：1，2，3，4"
        data.error_code = cons.BSERR_PARAM_ERR
    year = str(year)
    if not year.isdigit():
        print("年份输入有误，请修改。")
        data.error_msg = "年份输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_dupont_data", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, year, quarter)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYDUPONTDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYDUPONTDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.year = body_arr[8]
        data.quarter = body_arr[9]
        data.setFields(body_arr[10])
    return data


def query_balance_data(code, year=None, quarter=None):
    """季频偿债能力
    @param code: 证券代码，不可为空
    @param year: 统计年份，为空时默认当前年
    @param quarter: 统计季度，为空时默认当前季度
    """
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if year is None or year == "":
        year = time.strftime("%Y", time.localtime())  # 当前年份
    if quarter is None or quarter == "":
        quarter = (int(time.strftime("%m", time.localtime())) + 2) // 3  # 当前季度
    elif quarter not in [1, 2, 3, 4] and quarter not in ["1", "2", "3", "4"]:
        print("季度填写错误，请检查。填写范围：1，2，3，4")
        data.error_msg = "季度填写错误，请检查。填写范围：1，2，3，4"
        data.error_code = cons.BSERR_PARAM_ERR
    year = str(year)
    if not year.isdigit():
        print("年份输入有误，请修改。")
        data.error_msg = "年份输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_balance_data", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, year, quarter)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYBALANCEDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYBALANCEDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.year = body_arr[8]
        data.quarter = body_arr[9]
        data.setFields(body_arr[10])
    return data


def query_cash_flow_data(code, year=None, quarter=None):
    """季频现金流量
    @param code: 证券代码，不可为空
    @param year: 统计年份，为空时默认当前年
    @param quarter: 统计季度，为空时默认当前季度
    """
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if year is None or year == "":
        year = time.strftime("%Y", time.localtime())  # 当前年份
    if quarter is None or quarter == "":
        quarter = (int(time.strftime("%m", time.localtime())) + 2) // 3  # 当前季度
    elif quarter not in [1, 2, 3, 4] and quarter not in ["1", "2", "3", "4"]:
        print("季度填写错误，请检查。填写范围：1，2，3，4")
        data.error_msg = "季度填写错误，请检查。填写范围：1，2，3，4"
        data.error_code = cons.BSERR_PARAM_ERR
    year = str(year)
    if not year.isdigit():
        print("年份输入有误，请修改。")
        data.error_msg = "年份输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_cash_flow_data", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, year, quarter)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYCASHFLOWDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYCASHFLOWDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.year = body_arr[8]
        data.quarter = body_arr[9]
        data.setFields(body_arr[10])
    return data



import baostock as bs
import pandas as pd

#### 登陆系统 ####
lg = bs.login()
# 显示登陆返回信息
print('login respond error_code:'+lg.error_code)
print('login respond  error_msg:'+lg.error_msg)

#### 获取指数(综合指数、规模指数、一级行业指数、二级行业指数、策略指数、成长指数、价值指数、主题指数)K线数据 ####
##综合指数，例如：sh.000001 上证指数，sz.399106 深证综指 等；
##规模指数，例如：sh.000016 上证50，sh.000300 沪深300，sh.000905 中证500，sz.399001 深证成指等；
##一级行业指数，例如：sh.000037 上证医药，sz.399433 国证交运 等；
##二级行业指数，例如：sh.000952 300地产，sz.399951 300银行 等；
##策略指数，例如：sh.000050 50等权，sh.000982 500等权 等；
##成长指数，例如：sz.399376 小盘成长 等；
##价值指数，例如：sh.000029 180价值 等；
##主题指数，例如：sh.000015 红利指数，sh.000063 上证周期 等；

# 详细指标参数，参见“历史行情指标参数”章节
rs = bs.query_history_k_data("sh.600000",
    "date,code,open,high,low,close,preclose,volume,amount,pctChg",
    start_date='2017-01-01', end_date='2017-06-30', 
    frequency="d", adjustflag="3")
print('query_history_k_data respond error_code:'+rs.error_code)
print('query_history_k_data respond  error_msg:'+rs.error_msg)

#### 打印结果集 ####
data_list = []
while (rs.error_code == '0') & rs.next():
    # 获取一条记录，将记录合并在一起
    data_list.append(rs.get_row_data())
result = pd.DataFrame(data_list, columns=rs.fields)
#### 结果集输出到csv文件 ####   
result.to_csv("D:\\history_Index_k_data.csv", index=False)
print(result)

#### 登出系统 ####
bs.logout()


import baostock as bs
import pandas as pd

# 登陆系统
lg = bs.login(user_id="anonymous", password="123456")
# 显示登陆返回信息
print('login respond error_code:'+lg.error_code)
print('login respond  error_msg:'+lg.error_msg)

# 查询杜邦指数
dupont_list = []
rs_dupont = bs.query_dupont_data(code="sh.600000", year=2017, quarter=2)
while (rs_dupont.error_code == '0') & rs_dupont.next():
    dupont_list.append(rs_dupont.get_row_data())
result_profit = pd.DataFrame(dupont_list, columns=rs_dupont.fields)
# 打印输出
print(result_profit)
# 结果集输出到csv文件
result_profit.to_csv("D:\\dupont_data.csv", encoding="gbk", index=False)

# 登出系统
bs.logout()



# -*- coding:utf-8 -*-
"""
获行业分类
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""

import zlib
import baostock.data.resultset as rs
import baostock.common.contants as cons
import baostock.util.stringutil as strUtil
import baostock.common.context as conx
import baostock.util.socketutil as sock
import baostock.data.messageheader as msgheader


def query_stock_industry(code="", date=""):
    """获取行业分类
    @param code：股票代码，默认为空。
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if code is None or code == "":
        code = ""
    if code != "" and code is not None:
        if len(code) != cons.STOCK_CODE_LENGTH:
            print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
            data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
            data.error_code = cons.BSERR_PARAM_ERR
            return data
        code = code.lower()
        if (code.endswith("sh") or code.endswith("sz")):
            code = code[7:9].lower() + "." + code[0:6]

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx,"user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_stock_industry," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(code) + "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSTOCKINDUSTRY_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSTOCKINDUSTRY_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_hs300_stocks(date=""):
    """获取沪深300成分股
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_hs300_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYHS300STOCKS_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYHS300STOCKS_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data


def query_sz50_stocks(date=""):
    """获取上证50成分股
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data    
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_sz50_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSZ50STOCKS_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSZ50STOCKS_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data


def query_zz500_stocks(date=""):
    """获取中证500成分股
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_zz500_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYZZ500STOCKS_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYZZ500STOCKS_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_terminated_stocks(date=""):
    """获取终止上市股票列表
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_terminated_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYTERMINATEDSTOCKS_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYTERMINATEDSTOCKS_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_suspended_stocks(date=""):
    """获取暂停上市股票列表
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_suspended_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSUSPENDEDSTOCKS_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSUSPENDEDSTOCKS_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_st_stocks(date=""):
    """获取ST股票列表
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_st_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSTSTOCKS_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSTSTOCKS_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_starst_stocks(date=""):
    """获取*ST股票列表
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_starst_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSTARSTSTOCKS_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSTARSTSTOCKS_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data


def query_stock_concept(code="", date=""):
    """获取概念分类
    @param code：股票代码，默认为空。
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if code is None or code == "":
        code = ""
    if code != "" and code is not None:
        if len(code) != cons.STOCK_CODE_LENGTH:
            print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
            data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
            data.error_code = cons.BSERR_PARAM_ERR
            return data
        code = code.lower()
        if (code.endswith("sh") or code.endswith("sz")):
            code = code[7:9].lower() + "." + code[0:6]

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx,"user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_stock_concept," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(code) + "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSTOCKCONCEPT_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSTOCKCONCEPT_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.date = body_arr[8]
        data.setFields(body_arr[9])

    return data

def query_stock_area(code="", date=""):
    """获取地域分类
    @param code：股票代码，默认为空。
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if code is None or code == "":
        code = ""
    if code != "" and code is not None:
        if len(code) != cons.STOCK_CODE_LENGTH:
            print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
            data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
            data.error_code = cons.BSERR_PARAM_ERR
            return data
        code = code.lower()
        if (code.endswith("sh") or code.endswith("sz")):
            code = code[7:9].lower() + "." + code[0:6]

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx,"user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_stock_area," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(code) + "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSTOCKAREA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSTOCKAREA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_ame_stocks(date=""):
    """获取中小板分类
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_ame_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYAMESTOCK_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYAMESTOCK_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_gem_stocks(date=""):
    """获取创业板分类
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_gem_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYGEMSTOCK_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYGEMSTOCK_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_shhk_stocks(date=""):
    """获取沪港通股票
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
        
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_shhk_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSHHKSTOCK_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSHHKSTOCK_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_szhk_stocks(date=""):
    """获取深港通股票
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_szhk_stocks," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSZHKSTOCK_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSZHKSTOCK_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data

def query_stocks_in_risk(date=""):
    """获取风险警示板分类
    @param date：查询日期，默认为空。不为空时，格式 XXXX-XX-XX。
    """
    data = rs.ResultData()

    if date is None or date == "":
        date = ""
    else:
        if strUtil.is_valid_date(date):
            pass
        else:
            print("日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "日期格式不正确，请修改。"
            return data
    user_id = getattr(conx, "user_id")
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_stocks_in_risk," + str(user_id) + ",1," + \
            str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
            "," + str(date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSTOCKINRISK_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSTOCKINRISK_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.date = body_arr[7]
        data.setFields(body_arr[8])

    return data



# -*- coding:utf-8 -*-
"""
    证券系统元方法，如获取交易日等
"""
import time
import zlib
import baostock.data.resultset as rs
import baostock.common.contants as cons
import baostock.util.stringutil as strUtil
import baostock.common.context as conx
import baostock.util.socketutil as sock
import baostock.data.messageheader as msgheader


def query_trade_dates(start_date=None, end_date=None):
    """查询出给定范围的交易日信息

    @param start_date: 起始日期，默认2015-01-01
    @param end_date: 终止日期，默认当前日期
    @return: calendar_date 日期；is_trading_day，是否交易日，0:非交易日;1:交易日
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = cons.DEFAULT_START_DATE
    if end_date is None or end_date == "":
        end_date = time.strftime("%Y-%m-%d", time.localtime())

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s" % (
        "query_trade_dates", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, start_date, end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYTRADEDATES_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYTRADEDATES_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_all_stock(day=None):
    """查询给定日期的所有证券信息，

    @param day: 默认当前日期
    """
    data = rs.ResultData()

    if day is None or day == "":
        day = time.strftime("%Y-%m-%d", time.localtime())

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s" % (
        "query_all_stock", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, day)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYALLSTOCK_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYALLSTOCK_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.day = body_arr[7]
        data.setFields(body_arr[8])

    return data


def query_stock_basic(code="", code_name=""):
    """A股证券基本资料
    @param code: 证券代码，可为空
    @param code_name: 证券名称，可为空，支持模糊查询
    """
    data = rs.ResultData()

    if code is None or code == "":
        code = ""
    if code != "" and code is not None:
        if len(code) != cons.STOCK_CODE_LENGTH:
            print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
            data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
            data.error_code = cons.BSERR_PARAM_ERR
            return data
        code = code.lower()
        if (code.endswith("sh") or code.endswith("sz")):
            code = code[7:9].lower() + "." + code[0:6]

    if code_name is None or code_name == "":
        code_name = ""

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_stock_basic," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(code) + "," + str(code_name)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSTOCKBASIC_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSTOCKBASIC_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.code_name = body_arr[8]
        data.setFields(body_arr[9])

    return data



# -*- coding:utf-8 -*-
"""
常量类
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
# 版本信息
BAOSTOCK_CLIENT_VERSION = "00.8.80"
BAOSTOCK_AUTHOR = "baostock.com"
BAOSTOCK_SERVER_IP = "www.baostock.com"  # localhost  www.baostock.com  10.25.7.4
BAOSTOCK_SERVER_PORT = 10030

BAOSTOCK_SERVER_REAL_TIME_IP = "www.baostock.com"  # 实时行情服务地址 localhost  www.baostock.com
BAOSTOCK_SERVER_REAL_TIME_PORT = 10032  # 实时行情端口

BAOSTOCK_PER_PAGE_COUNT = 10000  # 默认每页查询条数

STOCK_CODE_LENGTH = 9  # 证券代码的长度
MESSAGE_SPLIT = "\1"  # 消息内部的分隔符

DELIMITER = "\n"  # 分隔符号,消息与消息之间的分隔符

# 参数各属性间的分隔符，如queryHistoryKData中的fields
ATTRIBUTE_SPLIT = ","
# 消息头中，消息体长度，占的位数
MESSAGE_HEADER_BODYLENGTH = 10
# message头长度
MESSAGE_HEADER_LENGTH = 21

BAOSTOCK_REALTIME_LIMIT_COUNT = 500  # 实时行情中，证券代码限制个数

# 以下为消息请求类型

MESSAGE_TYPE_LOGIN_REQUEST = "00"  # 登陆请求
MESSAGE_TYPE_LOGIN_RESPONSE = "01"  # 登陆响应
MESSAGE_TYPE_LOGOUT_REQUEST = "02"  # 登出请求
MESSAGE_TYPE_LOGOUT_RESPONSE = "03"  # 登出响应
MESSAGE_TYPE_EXCEPTIONS = "04"  # 错误信息
MESSAGE_TYPE_GETKDATA_REQUEST = "11"  # 获取历史K线数据请求
MESSAGE_TYPE_GETKDATA_RESPONSE = "12"  # 获取历史K线数据响应
MESSAGE_TYPE_QUERYDIVIDENDDATA_REQUEST = "13"  # 估值指标（季频）,股息分红 请求
MESSAGE_TYPE_QUERYDIVIDENDDATA_RESPONSE = "14"  # 估值指标（季频）,股息分红 响应
MESSAGE_TYPE_ADJUSTFACTOR_REQUEST = "15"  # 复权因子数据 请求
MESSAGE_TYPE_ADJUSTFACTOR_RESPONSE = "16"  # 复权因子数据 响应
MESSAGE_TYPE_PROFITDATA_REQUEST = "17"  # 季频估值指标盈利能力 请求
MESSAGE_TYPE_PROFITDATA_RESPONSE = "18"  # 季频估值指标盈利能力 响应
MESSAGE_TYPE_OPERATIONDATA_REQUEST = "19"  # 获取季频估值指标营运能力 请求
MESSAGE_TYPE_OPERATIONDATA_RESPONSE = "20"  # 获取季频估值指标营运能力 响应
MESSAGE_TYPE_QUERYGROWTHDATA_REQUEST = "21"  # 季频估值指标成长能力 请求
MESSAGE_TYPE_QUERYGROWTHDATA_RESPONSE = "22"  # 季频估值指标成长能力 响应
MESSAGE_TYPE_QUERYDUPONTDATA_REQUEST = "23"  # 获取季频估值指标杜邦指数  请求
MESSAGE_TYPE_QUERYDUPONTDATA_RESPONSE = "24"  # 获取季频估值指标杜邦指数  响应
MESSAGE_TYPE_QUERYBALANCEDATA_REQUEST = "25"  # 获取季频估值指标偿债能力  请求
MESSAGE_TYPE_QUERYBALANCEDATA_RESPONSE = "26"  # 获取季频估值指标偿债能力  响应
MESSAGE_TYPE_QUERYCASHFLOWDATA_REQUEST = "27"  # 获取季频估值指标现金流量   请求
MESSAGE_TYPE_QUERYCASHFLOWDATA_RESPONSE = "28"  # 获取季频估值指标现金流量   响应
MESSAGE_TYPE_QUERYPERFORMANCEEXPRESSREPORT_REQUEST = "29"  # 公司公告,公司业绩报告 请求
MESSAGE_TYPE_QUERYPERFORMANCEEXPRESSREPORT_RESPONSE = "30"  # 公司公告,公司业绩报告 响应
MESSAGE_TYPE_QUERYFORECASTREPORT_REQUEST = "31"  # 公司公告,公司业绩预告 请求
MESSAGE_TYPE_QUERYFORECASTREPORT_RESPONSE = "32"  # 公司公告,公司业绩预告 响应
MESSAGE_TYPE_QUERYTRADEDATES_REQUEST = "33"  # 查询出给定范围的交易日信息 请求
MESSAGE_TYPE_QUERYTRADEDATES_RESPONSE = "34"  # 查询出给定范围的交易日信息 响应
MESSAGE_TYPE_QUERYALLSTOCK_REQUEST = "35"  # 询给定日期的所有证券信息 请求
MESSAGE_TYPE_QUERYALLSTOCK_RESPONSE = "36"  # 询给定日期的所有证券信息 响应

MESSAGE_TYPE_LOGIN_REAL_TIME_REQUEST = "37"  # 实时行情登陆请求
MESSAGE_TYPE_LOGIN_REAL_TIME_RESPONSE = "38"  # 实时行情登陆响应
MESSAGE_TYPE_LOGOUT_REAL_TIME_REQUEST = "39"  # 实时行情登出请求
MESSAGE_TYPE_LOGOUT_REAL_TIME_RESPONSE = "40"  # 实时行情登出响应
MESSAGE_TYPE_SUBSCRIPTIONS_BY_SECURITYID_REQUEST = "41"  # 订阅行情消息请求
MESSAGE_TYPE_SUBSCRIPTIONS_BY_SECURITYID_RESPONSE = "42"  # 订阅行情消息响应
MESSAGE_TYPE_CANCEL_SUBSCRIBE_REQUEST = "43"  # 取消订阅行情消息请求
MESSAGE_TYPE_CANCEL_SUBSCRIBE_RESPONSE = "44"  # 取消订阅行情消息响应

MESSAGE_TYPE_QUERYSTOCKBASIC_REQUEST = "45"  # 获取证券基本资料请求
MESSAGE_TYPE_QUERYSTOCKBASIC_RESPONSE = "46"  # 获取证券基本资料响应
MESSAGE_TYPE_QUERYDEPOSITRATEDATA_REQUEST = "47"  # 获取存款利率请求
MESSAGE_TYPE_QUERYDEPOSITRATEDATA_RESPONSE = "48"  # 获取存款利率响应
MESSAGE_TYPE_QUERYLOANRATEDATA_REQUEST = "49"  # 获取贷款利率请求
MESSAGE_TYPE_QUERYLOANRATEDATA_RESPONSE = "50"  # 获取贷款利率响应
MESSAGE_TYPE_QUREYREQUIREDRESERVERATIODATA_REQUEST = "51"  # 获取存款准备金率请求
MESSAGE_TYPE_QUREYREQUIREDRESERVERATIODATA_RESPONSE = "52"  # 获取存款准备金率响应
MESSAGE_TYPE_QUERYMONEYSUPPLYDATAMONTH_REQUEST = "53"  # 获取货币供应量请求
MESSAGE_TYPE_QUERYMONEYSUPPLYDATAMONTH_RESPONSE = "54"  # 获取货币供应量响应
MESSAGE_TYPE_QUERYMONEYSUPPLYDATAYEAR_REQUEST = "55"  # 获取货币供应量（年底余额）请求
MESSAGE_TYPE_QUERYMONEYSUPPLYDATAYEAR_RESPONSE = "56"  # 获取货币供应量（年底余额）响应
MESSAGE_TYPE_QUERYSHIBORDATA_REQUEST = "57"  # 获取银行间同业拆放利率请求
MESSAGE_TYPE_QUERYSHIBORDATA_RESPONSE = "58"  # 获取银行间同业拆放利率响应
MESSAGE_TYPE_QUERYSTOCKINDUSTRY_REQUEST = "59"  # 获取行业类别请求
MESSAGE_TYPE_QUERYSTOCKINDUSTRY_RESPONSE = "60"  # 获取行业类别响应
MESSAGE_TYPE_QUERYHS300STOCKS_REQUEST = "61"  # 获取沪深300成分股请求
MESSAGE_TYPE_QUERYHS300STOCKS_RESPONSE = "62"  # 获取沪深300成分股响应
MESSAGE_TYPE_QUERYSZ50STOCKS_REQUEST = "63"  # 获取上证50成分股请求
MESSAGE_TYPE_QUERYSZ50STOCKS_RESPONSE = "64"  # 获取上证50成分股响应
MESSAGE_TYPE_QUERYZZ500STOCKS_REQUEST = "65"  # 获取中证500成分股请求
MESSAGE_TYPE_QUERYZZ500STOCKS_RESPONSE = "66"  # 获取中证500成分股响应
MESSAGE_TYPE_QUERYTERMINATEDSTOCKS_REQUEST = "67"  # 获取终止上市股票请求
MESSAGE_TYPE_QUERYTERMINATEDSTOCKS_RESPONSE = "68"  # 获取终止上市股票响应
MESSAGE_TYPE_QUERYSUSPENDEDSTOCKS_REQUEST = "69"   # 获取暂停上市股票列表请求
MESSAGE_TYPE_QUERYSUSPENDEDSTOCKS_RESPONSE = "70"  # 获取暂停上市股票列表响应
MESSAGE_TYPE_QUERYSTSTOCKS_REQUEST = "71"  # 获取ST股票列表请求
MESSAGE_TYPE_QUERYSTSTOCKS_RESPONSE = "72"  # 获取ST股票列表响应
MESSAGE_TYPE_QUERYSTARSTSTOCKS_REQUEST = "73"  # 获取*ST股票列表请求
MESSAGE_TYPE_QUERYSTARSTSTOCKS_RESPONSE = "74"  # 获取*ST股票列表响应
MESSAGE_TYPE_QUERYCPIDATA_REQUEST = "75"  # 获取居民价格消费指数请求
MESSAGE_TYPE_QUERYCPIDATA_RESPONSE = "76"  # 获取居民价格消费指数响应
MESSAGE_TYPE_QUERYPPIDATA_REQUEST = "77"    # 获取工业品出厂价格指数请求
MESSAGE_TYPE_QUERYPPIDATA_RESPONST = "78"    # 获取工业品出厂价格指数响应
MESSAGE_TYPE_QUERYPMIDATA_REQUEST = "79"    # 获取采购经理人指数请求
MESSAGE_TYPE_QUERYPMIDATA_RESPONST = "80"    # 获取采购经理人指数响应
MESSAGE_TYPE_QUERYSTOCKCONCEPT_REQUEST = "81"    # 获取概念分类请求
MESSAGE_TYPE_QUERYSTOCKCONCEPT_RESPONST = "82"    # 获取概念分类响应
MESSAGE_TYPE_QUERYSTOCKAREA_REQUEST = "83"    # 获取地域分类请求
MESSAGE_TYPE_QUERYSTOCKAREA_RESPONST = "84"    # 获取地域分类响应
MESSAGE_TYPE_QUERYAMESTOCK_REQUEST = "85"    # 获取中小板分类请求
MESSAGE_TYPE_QUERYAMESTOCK_RESPONST = "86"    # 获取中小板分类响应
MESSAGE_TYPE_QUERYGEMSTOCK_REQUEST = "87"    # 获取创业板分类请求
MESSAGE_TYPE_QUERYGEMSTOCK_RESPONST = "88"    # 获取创业板分类响应
MESSAGE_TYPE_QUERYSHHKSTOCK_REQUEST = "89"    # 获取沪港通请求
MESSAGE_TYPE_QUERYSHHKSTOCK_RESPONST = "90"    # 获取沪港通响应
MESSAGE_TYPE_QUERYSZHKSTOCK_REQUEST = "91"    # 获取深港通请求
MESSAGE_TYPE_QUERYSZHKSTOCK_RESPONST = "92"    # 获取深港通响应
MESSAGE_TYPE_QUERYSTOCKINRISK_REQUEST = "93"    # 获取风险警示板分类请求
MESSAGE_TYPE_QUERYSTOCKINRISK_RESPONST = "94"    # 获取风险警示板分类响应
MESSAGE_TYPE_GETKDATAPLUS_REQUEST = "95"   # 获取历史K线数据Plus的请求，行情压缩
MESSAGE_TYPE_GETKDATAPLUS_RESPONSE = "96"  # 获取历史K线数据Plus的响应，行情压缩


# 以上为消息请求类型


DEFAULT_START_DATE = "2015-01-01"  # 默认开始时间

# 方法名及包名的对应关系
MESSAGE_PACKAGE_MAPPING = {
    "__query_history_k_data_page": "baostock.security.history"

}


# 返回消息体进行压缩的响应代码
# 目前消息体进行压缩的有：获取历史K线数据响应
COMPRESSED_MESSAGE_TYPE_TUPLE = (MESSAGE_TYPE_GETKDATAPLUS_RESPONSE, )

# 以下是错误代码

BSERR_SUCCESS = "0"  # 正确返回值
BSERR_NO_LOGIN = "10001001"  # 用户未登陆
BSERR_USERNAMEORPASSWORD_ERR = "10001002"  # 用户名或密码错误
BSERR_GETUSERINFO_FAIL = "10001003"  # 获取用户信息失败
BSERR_CLIENT_VESION_EXPIRE = "10001004"  # 客户端版本号过期
BSERR_LOGIN_COUNT_LIMIT = "10001005"  # 账号登陆数达到上限
BSERR_ACCESS_INSUFFICIENCE = "10001006"  # 用户权限不足
BSERR_NEED_ACTIVATE = "10001007"  # 需要登录激活
BSERR_USERNAME_EMPTY = "10001008"  # 用户名为空
BSERR_PASSWORD_EMPTY = "10001009"  # 密码为空
BSERR_LOGOUT_FAIL = "10001010"  # 用户登出失败
BSERR_BLACKLIST_USER = "10001011"  # 黑名单用户
BSERR_SOCKET_ERR = "10002001"  # 网络错误
BSERR_CONNECT_FAIL = "10002002"  # 网络连接失败
BSERR_CONNECT_TIMEOUT = "10002003"  # 网络连接超时
BSERR_RECVCONNECTION_CLOSED = "10002004"  # 网络接收时连接断开
BSERR_SENDSOCK_FAIL = "10002005"  # 网络发送失败
BSERR_SENDSOCK_TIMEOUT = "10002006"  # 网络发送超时
BSERR_RECVSOCK_FAIL = "10002007"  # 网络接收错误
BSERR_RECVSOCK_TIMEOUT = "10002008"  # 网络接收超时
BSERR_PARSE_DATA_ERR = "10004001"  # 解析数据错误
BSERR_UNGZIP_DATA_FAIL = "10004002"  # gzip 解压失败
BSERR_UNKNOWN_ERR = "10004003"  # 客户端未知错误
BSERR_OUTOF_BOUNDS = "10004004"  # 数组越界
BSERR_INPARAM_EMPTY = "10004005"  # 传入参数为空
BSERR_PARAM_ERR = "10004006"  # 参数错误
BSERR_START_DATE_ERR = "10004007"  # 起始日期格式不正确
BSERR_END_DATE_ERR = "10004008"  # 截止日期格式不正确
BSERR_START_BIGTHAN_END = "10004009"  # 起始日期大于终止日期
BSERR_DATE_ERR = "10004010"  # 日期格式不正确
BSERR_CODE_INVALIED = "10004011"  # 无效的证券代码
BSERR_INDICATOR_INVALIED = "10004012"  # 无效的指标
BSERR_BEYOND_DATE_SUPPORT = "10004013"  # 超出日期支持范围
BSERR_MIXED_CODES_MARKET = "10004014"  # 不支持的混合证券品种
BSERR_NO_SUPPORT_CODES_MARKET = "10004015"  # 不支持的证券代码品种
BSERR_ORDER_TO_UPPER_LIMIT = "10004016"  # 交易条数超过上限
BSERR_NO_SUPPORT_ORDERINFO = "10004017"  # 不支持的交易信息
BSERR_INDICATOR_REPEAT = "10004018"  # 指标重复
BSERR_MESSAGE_ERROR = "10004019"  # 消息格式不正确
BSERR_MESSAGE_CODE_ERROR = "10004020"  # 错误的消息类型
BSERR_SYSTEM_ERROR = "10005001"  # 系统级别错误

# 以上是错误代码


CLIENT_ERROR_PARAM = "参数错误，请检查。"  # 客户端参数错误



'''
Created on 2019年1月17日

@author: ZYP
'''
import baostock as bs
import pandas as pd



def test_day_data(code, adjust_flag):
    #### 获取历史K线数据 ####
    # 详细指标参数，参见“历史行情指标参数”章节
        
    rs_old = bs.query_history_k_data(code,
    "date,code,open,high,low,close,preclose,volume,amount,adjustflag,turn,tradestatus,pctChg,peTTM,pbMRQ,psTTM,pcfNcfTTM,isST",
    start_date='1990-01-01', end_date='2019-01-16', 
    frequency="d", adjustflag=adjust_flag)
#### 打印结果集 ####
    data_list = []
    while (rs_old.error_code == '0') & rs_old.next():
        # 获取一条记录，将记录合并在一起
        data_list.append(rs_old.get_row_data())
    result1 = pd.DataFrame(data_list, columns=rs_old.fields)
    #### 结果集输出到csv文件 ####
#         result1.to_csv("D:/history_k_data_plus.csv", encoding="gbk", index=False)
    
    rs_old_2 = bs.query_history_k_data(code,
        "date,code,open,high,low,close,preclose,volume,amount,adjustflag,turn,tradestatus,pctChg,peTTM,pbMRQ,psTTM,pcfNcfTTM,isST",
        start_date='1990-01-01', end_date='2019-01-16', 
         frequency='d', adjustflag=adjust_flag)
    result2 = rs_old_2.get_data();
#         result2.to_csv("D:/history_k_data.csv", encoding="gbk", index=False)

    rs_plus = bs.query_history_k_data_plus(code,
    "date,code,open,high,low,close,preclose,volume,amount,adjustflag,turn,tradestatus,pctChg,peTTM,pbMRQ,psTTM,pcfNcfTTM,isST",
    start_date='1990-01-01', end_date='2019-01-16', 
     frequency='d', adjustflag=adjust_flag)
    #### 打印结果集 ####
    data_list = []
    while (rs_plus.error_code == '0') & rs_plus.next():
        # 获取一条记录，将记录合并在一起
        data_list.append(rs_plus.get_row_data())
    result3 = pd.DataFrame(data_list, columns=rs_plus.fields)
    #### 结果集输出到csv文件 ####
#         result3.to_csv("D:/history_k_data_plus.csv", encoding="gbk", index=False)
    
    rs_plus_2 = bs.query_history_k_data_plus(code,
        "date,code,open,high,low,close,preclose,volume,amount,adjustflag,turn,tradestatus,pctChg,peTTM,pbMRQ,psTTM,pcfNcfTTM,isST",
        start_date='1990-01-01', end_date='2019-01-16', 
         frequency='d', adjustflag=adjust_flag)
    result4 = rs_plus_2.get_data();
    #### 结果集输出到csv文件 ####
#         result4.to_csv("D:/history_k_data_plus2.csv", encoding="gbk", index=False)

#     print(code + " test finished!")
#     fileopen.write(code + " test finished!")
        
def test_week_month_data(code, frequency_flag, adjust_flag):
    #### 获取历史K线数据 ####
    # 详细指标参数，参见“历史行情指标参数”章节
        
    rs_old = bs.query_history_k_data(code,
    "date,code,open,high,low,close,volume,amount,adjustflag,turn,pctChg",
    start_date='1990-01-01', end_date='2019-01-16', 
    frequency=frequency_flag, adjustflag=adjust_flag)
#     print('query_history_k_data respond error_code:'+rs.error_code)
#     print('query_history_k_data respond  error_msg:'+rs.error_msg)
#### 打印结果集 ####
    data_list = []
    while (rs_old.error_code == '0') & rs_old.next():
        # 获取一条记录，将记录合并在一起
        data_list.append(rs_old.get_row_data())
    result1 = pd.DataFrame(data_list, columns=rs_old.fields)
    #### 结果集输出到csv文件 ####
#         result1.to_csv("D:/history_k_data_plus.csv", encoding="gbk", index=False)
    
    
    rs_old_2 = bs.query_history_k_data(code,
        "date,code,open,high,low,close,volume,amount,adjustflag,turn,pctChg",
        start_date='1990-01-01', end_date='2019-01-16', 
         frequency=frequency_flag, adjustflag=adjust_flag)
    result2 = rs_old_2.get_data();
#         result2.to_csv("D:/history_k_data.csv", encoding="gbk", index=False)
    
    
    
    
    rs_plus = bs.query_history_k_data_plus(code,
    "date,code,open,high,low,close,volume,amount,adjustflag,turn,pctChg",
    start_date='1990-01-01', end_date='2019-01-16', 
     frequency=frequency_flag, adjustflag=adjust_flag)
    #### 打印结果集 ####
    data_list = []
    while (rs_plus.error_code == '0') & rs_plus.next():
        # 获取一条记录，将记录合并在一起
        data_list.append(rs_plus.get_row_data())
    result3 = pd.DataFrame(data_list, columns=rs_plus.fields)
    #### 结果集输出到csv文件 ####
#         result3.to_csv("D:/history_k_data_plus.csv", encoding="gbk", index=False)
    
    
    rs_plus_2 = bs.query_history_k_data_plus(code,
        "date,code,open,high,low,close,volume,amount,adjustflag,turn,pctChg",
        start_date='1990-01-01', end_date='2019-01-16', 
         frequency=frequency_flag, adjustflag=adjust_flag)
#     print('query_history_k_data respond error_code:'+rs.error_code)
#     print('query_history_k_data respond  error_msg:'+rs.error_msg)
    result4 = rs_plus_2.get_data();
    #### 结果集输出到csv文件 ####
#         result4.to_csv("D:/history_k_data_plus2.csv", encoding="gbk", index=False)

#     print(code + " test finished!")
#     fileopen.write(code + " test finished!")

def test_minutes_data(code, frequency_flag, adjust_flag):
    #### 获取历史K线数据 ####
    # 详细指标参数，参见“历史行情指标参数”章节
        
    rs_old = bs.query_history_k_data(code,
    "date,code,open,high,low,close,volume,amount,adjustflag",
    start_date='1990-01-01', end_date='2019-01-16', 
    frequency=frequency_flag, adjustflag=adjust_flag)
#     print('query_history_k_data respond error_code:'+rs.error_code)
#     print('query_history_k_data respond  error_msg:'+rs.error_msg)
#### 打印结果集 ####
    data_list = []
    while (rs_old.error_code == '0') & rs_old.next():
        # 获取一条记录，将记录合并在一起
        data_list.append(rs_old.get_row_data())
    result1 = pd.DataFrame(data_list, columns=rs_old.fields)
    #### 结果集输出到csv文件 ####
#         result1.to_csv("D:/history_k_data_plus.csv", encoding="gbk", index=False)
    
    
    rs_old_2 = bs.query_history_k_data(code,
        "date,code,open,high,low,close,volume,amount,adjustflag",
        start_date='1990-01-01', end_date='2019-01-16', 
         frequency=frequency_flag, adjustflag=adjust_flag)
    result2 = rs_old_2.get_data();
#         result2.to_csv("D:/history_k_data.csv", encoding="gbk", index=False)
    
    
    
    
    rs_plus = bs.query_history_k_data_plus(code,
    "date,code,open,high,low,close,volume,amount,adjustflag",
    start_date='1990-01-01', end_date='2019-01-16', 
     frequency=frequency_flag, adjustflag=adjust_flag)
    #### 打印结果集 ####
    data_list = []
    while (rs_plus.error_code == '0') & rs_plus.next():
        # 获取一条记录，将记录合并在一起
        data_list.append(rs_plus.get_row_data())
    result3 = pd.DataFrame(data_list, columns=rs_plus.fields)
    #### 结果集输出到csv文件 ####
#         result3.to_csv("D:/history_k_data_plus.csv", encoding="gbk", index=False)
    
    
    rs_plus_2 = bs.query_history_k_data_plus(code,
        "date,code,open,high,low,close,volume,amount,adjustflag",
        start_date='1990-01-01', end_date='2019-01-16', 
         frequency=frequency_flag, adjustflag=adjust_flag)
#     print('query_history_k_data respond error_code:'+rs.error_code)
#     print('query_history_k_data respond  error_msg:'+rs.error_msg)
    result4 = rs_plus_2.get_data();
    #### 结果集输出到csv文件 ####
#         result4.to_csv("D:/history_k_data_plus2.csv", encoding="gbk", index=False)
# 
#     print(code + " test finished!")
#     fileopen.write(code + " test finished!")




if __name__ == '__main__':
    #### 登陆系统 ####
    lg = bs.login()
    # 显示登陆返回信息
    print('login respond error_code:'+lg.error_code)
    print('login respond  error_msg:'+lg.error_msg)
    
    #### 获取全部证券 ####
    codes = bs.query_stock_basic()
    codesdf = codes.get_data()
    codesdf.to_csv("D:/history_k_data_codes.csv", encoding="gbk", index=False)
    codeslist = codesdf['code']
    
    fileopen = open("D:/history.logs","a+")
    
    frequency_flag = "m"
#     adjust_flag = "3"
#     fileopen.write(" adjust_flag:"+adjust_flag+"\n")
    for code in codeslist:
        test_day_data(code,"3")
        test_week_month_data(code,"m","3")
        test_week_month_data(code,"w","3")
        test_minutes_data(code,"60","3")
        test_minutes_data(code,"30","3")
        test_minutes_data(code,"15","3")
        test_minutes_data(code,"5","3")

        fileopen.write(code + " 不复权  test finished!\n")
        
        test_day_data(code,"1")
        test_week_month_data(code,"m","1")
        test_week_month_data(code,"w","1")
        test_minutes_data(code,"60","1")
        test_minutes_data(code,"30","1")
        test_minutes_data(code,"15","1")
        test_minutes_data(code,"5","1")

        fileopen.write(code + " 前复权  test finished!\n")
        
        test_day_data(code,"2")
        test_week_month_data(code,"m","2")
        test_week_month_data(code,"w","2")
        test_minutes_data(code,"60","2")
        test_minutes_data(code,"30","2")
        test_minutes_data(code,"15","2")
        test_minutes_data(code,"5","2")
        print(code + " test finished!")
        fileopen.write(code + " 后复权 test finished!\n")
    fileopen.close()
    #### 登出系统 ####
    bs.logout()



# -*- coding:utf-8 -*-
"""
发布方法
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import baostock.common.contants as cons
__version__ = cons.BAOSTOCK_CLIENT_VERSION
__author__ = cons.BAOSTOCK_AUTHOR

# login/logout
from baostock.login.loginout import (login, logout)

# history data
from baostock.security.history import (query_history_k_data, query_history_k_data_plus)

# sector info
from baostock.security.sectorinfo import (query_stock_industry,
                                          query_hs300_stocks,
                                          query_sz50_stocks,
                                          query_zz500_stocks)

# evaluation data
from baostock.evaluation.season_index import (query_dividend_data, query_adjust_factor,
                                              query_profit_data, query_operation_data,
                                              query_growth_data, query_dupont_data,
                                              query_balance_data, query_cash_flow_data)

# corporate performance
from baostock.corpreport.corp_performance import (
    query_performance_express_report, query_forecast_report)

# metadata
from baostock.metadata.stock_metadata import (query_trade_dates, query_all_stock,
                                              query_stock_basic)

# macroscopic
from baostock.macroscopic.economic_data import (query_deposit_rate_data, query_loan_rate_data,
                                                query_required_reserve_ratio_data, query_money_supply_data_month,
                                                query_money_supply_data_year, query_shibor_data)



# -*- coding:utf-8 -*-
"""
大类：公司公告，小类：公司业绩报告
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
@copyright: baostock System & alpha.All Rights Reserved.
"""

import datetime
import time
import zlib

import baostock.data.resultset as rs
import baostock.common.contants as cons
import baostock.util.stringutil as strUtil
import baostock.common.context as conx
import baostock.util.socketutil as sock
import baostock.data.messageheader as msgheader


def query_performance_express_report(code, start_date=None, end_date=None):
    """公司业绩快报。
    @param code: 证券代码，不可为空
    @param start_date: 开始日期，默认2015-01-01；发布日期或更新日期在这个范围内。
    @param end_date: 结束日期，默认系统当前日期；发布日期或更新日期在这个范围内。
    """

    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if start_date is None or start_date == "":
        start_date = cons.DEFAULT_START_DATE
    if end_date is None or end_date == "":
        end_date = time.strftime("%Y-%m-%d", time.localtime())

    if start_date != "" and start_date is not None and end_date != "" and end_date is not None:
        if strUtil.is_valid_date(start_date) and strUtil.is_valid_date(end_date):
            start_date_time = datetime.datetime.strptime(
                start_date, '%Y-%m-%d')
            end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
            if end_date_time < start_date_time:
                print("起始日期大于终止日期，请修改。")
                data.error_code = cons.BSERR_START_BIGTHAN_END
                data.error_msg = "起始日期大于终止日期，请修改。"
                return data
        else:
            print("日期格式不正确，请修改。")
            return

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_performance_express_report", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, start_date, end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYPERFORMANCEEXPRESSREPORT_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYPERFORMANCEEXPRESSREPORT_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.start_date = body_arr[8]
        data.end_date = body_arr[9]
        data.setFields(body_arr[10])

    return data


def query_forecast_report(code, start_date=None, end_date=None):
    """公司业绩预告，
    @param code: 证券代码，不可为空
    @param start_date: 开始日期，默认2015-01-01；发布日期或统计日期在这个范围内。
    @param end_date: 结束日期，默认系统当前日期；发布日期或统计日期在这个范围内。
    """
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != cons.STOCK_CODE_LENGTH:
        print("股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。")
        data.error_msg = "股票代码应为" + str(cons.STOCK_CODE_LENGTH) + "位，请检查。格式示例：sh.600000。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    code = code.lower()
    if (code.endswith("sh") or code.endswith("sz")):
        code = code[7:9].lower() + "." + code[0:6]

    if start_date is None or start_date == "":
        start_date = cons.DEFAULT_START_DATE
    if end_date is None or end_date == "":
        end_date = time.strftime("%Y-%m-%d", time.localtime())

    if start_date != "" and start_date is not None and end_date != "" and end_date is not None:
        if strUtil.is_valid_date(start_date) and strUtil.is_valid_date(end_date):
            start_date_time = datetime.datetime.strptime(
                start_date, '%Y-%m-%d')
            end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
            if end_date_time < start_date_time:
                print("起始日期大于终止日期，请修改。")
                data.error_code = cons.BSERR_START_BIGTHAN_END
                data.error_msg = "起始日期大于终止日期，请修改。"
                return data
        else:
            print("日期格式不正确，请修改。")
            return

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "%s,%s,%s,%s,%s,%s,%s" % (
        "query_forecast_report", user_id, "1",
        cons.BAOSTOCK_PER_PAGE_COUNT, code, start_date, end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYFORECASTREPORT_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYFORECASTREPORT_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.code = body_arr[7]
        data.start_date = body_arr[8]
        data.end_date = body_arr[9]
        data.setFields(body_arr[10])

    return data



import baostock as bs


# 每次收到实时行情后，回调此方法
def callbackFunc(ResultData):
    print(ResultData.data)


if __name__ == '__main__':
    # 登陆
    login_result = bs.login_real_time()
    print('login respond error_code:' + login_result.error_code)
    print('login respond  error_msg:' + login_result.error_msg)
    # 订阅
    rs = bs.subscribe_by_code("sh.600000,sz.000001", 0, callbackFunc, "", "user_params")
    if rs.error_code != '0':
        print("request real time error", rs.error_msg)
    else:
        # 使主程序不再向下执行。使用time.sleep()等方法也可以
        text = input("press any key to cancel real time \r\n")
        # 取消订阅
        cancel_rs = bs.cancel_subscribe(rs.serial_id)
    # 登出
    login_result = bs.logout_real_time()



# -*- coding:utf-8 -*-
"""
宏观数据，宏观经济数据
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
@copyright: baostock System & alpha.All Rights Reserved.
"""
import time
import zlib
import datetime
import baostock.data.resultset as rs
import baostock.common.contants as cons
import baostock.util.stringutil as strUtil
import baostock.common.context as conx
import baostock.util.socketutil as sock
import baostock.data.messageheader as msgheader


def query_deposit_rate_data(start_date="", end_date=""):
    """存款利率
    @param sart_date: 起始日期，包含次此日期，可为空
    @param end_date: 结束日期，包含次此日期，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_date(start_date):
            pass
        else:
            print("起始日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_date(end_date):
            pass
        else:
            print("结束日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(
            start_date, '%Y-%m-%d')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_deposit_rate_data," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYDEPOSITRATEDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYDEPOSITRATEDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_loan_rate_data(start_date="", end_date=""):
    """贷款利率
    @param sart_date: 起始日期，包含次此日期，可为空
    @param end_date: 结束日期，包含次此日期，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_date(start_date):
            pass
        else:
            print("起始日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_date(end_date):
            pass
        else:
            print("结束日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(
            start_date, '%Y-%m-%d')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_loan_rate_data," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYLOANRATEDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYLOANRATEDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_required_reserve_ratio_data(start_date="", end_date="", yearType="0"):
    """存款准备金率
    @param sart_date: 起始日期，包含次此日期，可为空
    @param end_date: 结束日期，包含次此日期，可为空
    @param yearType: 日期类型，默认0为公告日期，1为生效日期
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_date(start_date):
            pass
        else:
            print("起始日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_date(end_date):
            pass
        else:
            print("结束日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(
            start_date, '%Y-%m-%d')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    if yearType is None or yearType == "":
        print("年份类别输入有误，请修改。")
        data.error_msg = "年份类别输入有误，请修改。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_required_reserve_ratio_data," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date) + "," + str(yearType)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUREYREQUIREDRESERVERATIODATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUREYREQUIREDRESERVERATIODATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.yearType = body_arr[9]
        data.setFields(body_arr[10])

    return data


def query_money_supply_data_month(start_date="", end_date=""):
    """货币供应量
    @param sart_date: 起始年月yyyy-MM，包含次此日期，可为空
    @param end_date: 结束年月yyyy-MM，包含次此日期，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_year_month_date(start_date):
            pass
        else:
            print("起始日期格式不正确，应为：yyyy-mm，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，应为：yyyy-mm，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_year_month_date(end_date):
            pass
        else:
            print("结束日期格式不正确，应为：yyyy-mm，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，应为：yyyy-mm，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(start_date, '%Y-%m')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_money_supply_data_month," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYMONEYSUPPLYDATAMONTH_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYMONEYSUPPLYDATAMONTH_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_money_supply_data_year(start_date="", end_date=""):
    """货币供应量（年底余额）
    @param sart_date: 起始年份yyyy，包含次此年份，可为空
    @param end_date: 结束年份yyyy，包含次此年份，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_year_date(start_date):
            pass
        else:
            print("起始日期格式不正确，应为：yyyy，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，应为：yyyy，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_year_date(end_date):
            pass
        else:
            print("结束日期格式不正确，应为：yyyy，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，应为：yyyy，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(start_date, '%Y')
        end_date_time = datetime.datetime.strptime(end_date, '%Y')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_money_supply_data_year," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYMONEYSUPPLYDATAYEAR_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYMONEYSUPPLYDATAYEAR_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_shibor_data(start_date="", end_date=""):
    """银行间同业拆放利率
    @param sart_date: 起始日期，包含次此日期，可为空
    @param end_date: 结束日期，包含次此日期，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_date(start_date):
            pass
        else:
            print("起始日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_date(end_date):
            pass
        else:
            print("结束日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(
            start_date, '%Y-%m-%d')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_shibor_data," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYSHIBORDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYSHIBORDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_cpi_data(start_date="", end_date=""):
    """居民价格消费指数
    @param sart_date: 起始日期，包含次此日期，可为空
    @param end_date: 结束日期，包含次此日期，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_date(start_date):
            pass
        else:
            print("起始日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_date(end_date):
            pass
        else:
            print("结束日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(
            start_date, '%Y-%m-%d')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_cpi_data," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYCPIDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYCPIDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_ppi_data(start_date="", end_date=""):
    """工业品出厂价格指数
    @param sart_date: 起始日期，包含次此日期，可为空
    @param end_date: 结束日期，包含次此日期，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_date(start_date):
            pass
        else:
            print("起始日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_date(end_date):
            pass
        else:
            print("结束日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(
            start_date, '%Y-%m-%d')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_ppi_data," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYPPIDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYPPIDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data


def query_pmi_data(start_date="", end_date=""):
    """采购经理人指数
    @param sart_date: 起始日期，包含次此日期，可为空
    @param end_date: 结束日期，包含次此日期，可为空
    """
    data = rs.ResultData()

    if start_date is None or start_date == "":
        start_date = ""
    else:
        if strUtil.is_valid_date(start_date):
            pass
        else:
            print("起始日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "起始日期格式不正确，请修改。"
            return data
    if end_date is None or end_date == "":
        end_date = ""
    else:
        if strUtil.is_valid_date(end_date):
            pass
        else:
            print("结束日期格式不正确，请修改。")
            data.error_code = cons.BSERR_DATE_ERR
            data.error_msg = "结束日期格式不正确，请修改。"
            return data
    if start_date != "" and end_date != "":
        start_date_time = datetime.datetime.strptime(
            start_date, '%Y-%m-%d')
        end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        if end_date_time < start_date_time:
            print("起始日期大于结束日期，请修改。")
            data.error_code = cons.BSERR_START_BIGTHAN_END
            data.error_msg = "起始日期大于结束日期，请修改。"
            return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("you don't login.")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "you don't login."
        return data

    param = "query_pmi_data," + str(user_id) + ",1," + \
        str(cons.BAOSTOCK_PER_PAGE_COUNT) + \
        "," + str(start_date) + "," + str(end_date)

    msg_body = strUtil.organize_msg_body(param)
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_QUERYPMIDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_QUERYPMIDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]
    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)
    data.msg_body_length = header_arr[2]
    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.setData(body_arr[6])
        data.start_date = body_arr[7]
        data.end_date = body_arr[8]
        data.setFields(body_arr[9])

    return data

